@@ -22,18 +22,19 @@ require Exporter;
 
 use strict;
 use vars qw[$DEBUG $error $VERSION $WARN $FOLLOW_SYMLINK $CHOWN $CHMOD
-            $DO_NOT_USE_PREFIX $HAS_PERLIO $HAS_IO_STRING
+            $DO_NOT_USE_PREFIX $HAS_PERLIO $HAS_IO_STRING $SAME_PERMISSIONS
             $INSECURE_EXTRACT_MODE @ISA @EXPORT
          ];
 
 @ISA                    = qw[Exporter];
-@EXPORT                 = ( COMPRESS_GZIP, COMPRESS_BZIP );
+@EXPORT                 = qw[ COMPRESS_GZIP COMPRESS_BZIP ];
 $DEBUG                  = 0;
 $WARN                   = 1;
 $FOLLOW_SYMLINK         = 0;
-$VERSION                = "1.42";
+$VERSION                = "1.50";
 $CHOWN                  = 1;
 $CHMOD                  = 1;
+$SAME_PERMISSIONS       = $> == 0 ? 1 : 0;
 $DO_NOT_USE_PREFIX      = 0;
 $INSECURE_EXTRACT_MODE  = 0;
 
@@ -68,8 +69,8 @@ Archive::Tar - module for manipulations of tar archives
     $tar->rename('oldname', 'new/file/name');
 
     $tar->write('files.tar');                   # plain tar
-    $tar->write('files.tgz', COMPRESSED_GZIP);  # gzip compressed
-    $tar->write('files.tbz', COMPRESSED_BZIP);  # bzip2 compressed
+    $tar->write('files.tgz', COMPRESS_GZIP);    # gzip compressed
+    $tar->write('files.tbz', COMPRESS_BZIP);    # bzip2 compressed
 
 =head1 DESCRIPTION
 
@@ -323,6 +324,13 @@ sub _read_tar {
                 $self->_error( qq[Cannot read compressed format in tar-mode] );
                 return;
             }
+            
+            ### size is < HEAD, which means a corrupted file, as the minimum
+            ### length is _at least_ HEAD
+            if (length $chunk != HEAD) {
+                $self->_error( qq[Cannot read enough bytes from the tarfile] );
+                return;
+            }
         }
 
         ### if we can't read in all bytes... ###
@@ -799,7 +807,11 @@ sub _extract_file {
     ### only chmod if we're allowed to, but never chmod symlinks, since they'll
     ### change the perms on the file they're linking too...
     if( $CHMOD and not -l $full ) {
-        chmod $entry->mode, $full or
+        my $mode = $entry->mode;
+        unless ($SAME_PERMISSIONS) {
+            $mode &= ~(oct(7000) | umask);
+        }
+        chmod $mode, $full or
             $self->_error( qq[Could not chown '$full' to ] . $entry->mode );
     }
 
@@ -1073,10 +1085,10 @@ to be the C<gzip> compression level (between 1 and 9), but the use of
 constants is prefered:
 
   # write a gzip compressed file
-  $tar->write( 'out.tgz', COMPRESSION_GZIP );
+  $tar->write( 'out.tgz', COMPRESS_GZIP );
 
   # write a bzip compressed file
-  $tar->write( 'out.tbz', COMPRESSION_BZIP );
+  $tar->write( 'out.tbz', COMPRESS_BZIP );
 
 Note that when you pass in a filehandle, the compression argument
 is ignored, as all files are printed verbatim to your filehandle.
@@ -1503,10 +1515,10 @@ to be the C<gzip> compression level (between 1 and 9), but the use of
 constants is prefered:
 
   # write a gzip compressed file
-  Archive::Tar->create_archive( 'out.tgz', COMPRESSION_GZIP, @filelist );
+  Archive::Tar->create_archive( 'out.tgz', COMPRESS_GZIP, @filelist );
 
   # write a bzip compressed file
-  Archive::Tar->create_archive( 'out.tbz', COMPRESSION_BZIP, @filelist );
+  Archive::Tar->create_archive( 'out.tbz', COMPRESS_BZIP, @filelist );
 
 Note that when you pass in a filehandle, the compression argument
 is ignored, as all files are printed verbatim to your filehandle.
@@ -1754,6 +1766,15 @@ variable to C<0> to disable C<chmod>-ing.
 
 The default is C<1>.
 
+=head2 $Archive::Tar::SAME_PERMISSIONS
+
+When, C<$Archive::Tar::CHMOD> is enabled, this setting controls whether
+the permissions on files from the archive are used without modification
+of if they are filtered by removing any setid bits and applying the
+current umask.
+
+The default is C<1> for the root user and C<0> for normal users.
+
 =head2 $Archive::Tar::DO_NOT_USE_PREFIX
 
 By default, C<Archive::Tar> will try to put paths that are over
